/*
 * LCD_2x16.c
 *
 *  Created on: Oct 17, 2023
 *      Author: JM PALOMINO
 */

#include "LCD_2x16.h"
/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "COPFree.h"
#include "PTB.h"
/* Include shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#define LCD_RD7     PTBD_PTBD7     //D7
#define LCD_RD6     PTBD_PTBD6     //D6
#define LCD_RD5     PTBD_PTBD5     //D5
#define LCD_RD4     PTBD_PTBD4     //D4
#define LCD_EN      PTBD_PTBD3     //EN
#define LCD_RS      PTBD_PTBD2     //RS


void delay_lcd(void) //aprox 5 us for cpu clock = 8MHz with internal clock source.
{  
 asm { 
	    CLR $82
	    MOV #$2, $82
	    b: 
	      NOP
	      DBNZ $82, b
	  }	
}

void Lcd_Init(void){
	
unsigned char data1;

LCD_RD7 = 0;
LCD_RD6 = 0;
LCD_RD5 = 0;
LCD_RD4 = 0;
LCD_EN = 0;
LCD_RS = 0;
Cpu_Delay100US(330);

for(data1 = 1; data1 < 4; data1 ++)
{
	
	    LCD_RD7 = 0;   LCD_RD6 = 0;    LCD_RD5 = 1;    LCD_RD4 = 1;    LCD_EN = 0;
	    LCD_RS = 0;    LCD_RD7 = 0;    LCD_RD6 = 0;    LCD_RD5 = 1;    LCD_RD4 = 1;
	    LCD_EN = 1;    LCD_RS = 0;     delay_lcd();    
	    LCD_RD7 = 0;   LCD_RD6 = 0;    LCD_RD5 = 1;    LCD_RD4 = 1;    LCD_EN = 0;
	    LCD_RS = 0;    Cpu_Delay100US(55);

	}

	LCD_RD7 = 0; LCD_RD6 = 0; LCD_RD5 = 1; LCD_RD4 = 0; LCD_EN = 0; LCD_RS = 0;
	LCD_RD7 = 0; LCD_RD6 = 0; LCD_RD5 = 1; LCD_RD4 = 0; LCD_EN = 1; LCD_RS = 0;
	delay_lcd();
	LCD_RD7 = 0; LCD_RD6 = 0; LCD_RD5 = 1; LCD_RD4 = 0; LCD_EN = 0; LCD_RS = 0;
	Cpu_Delay100US(55);

	Lcd_Cmd(LCD_FOR_2LINE);
	Lcd_Cmd(LCD_MOVE_CURSOR_RIGHT);
	Lcd_Cmd(LCD_CLEAR);
	Lcd_Cmd(LCD_DIRECTION);
	Lcd_Cmd(LCD_CURSOR_OFF);

}

void Lcd_Out(unsigned char P, unsigned char L, char *buffer)
{
unsigned char data1;
switch (P)
{
    case 1: data1 = 0x80 + L; break;
    case 2: data1 = 0xC0 + L; break;
    case 3: data1 = 0x94 + L; break;
    case 4: data1 = 0xD4 + L; break;
    default: break;
}
Lcd_Cmd(data1);
while(*buffer)              // Write data to LCD up to null
{                
    Lcd_Chr_CP(*buffer);
    buffer++;               // Increment buffer
}
return;
}

void Lcd_Chr_CP(char data1){
LCD_EN = 0; LCD_RS = 1;
LCD_RD7 = (data1 & 0b10000000)>>7; LCD_RD6 = (data1 & 0b01000000)>>6;
LCD_RD5 = (data1 & 0b00100000)>>5; LCD_RD4 = (data1 & 0b00010000)>>4;
Cpu_Delay100US(100);
LCD_EN = 1; delay_lcd(); LCD_EN = 0;
LCD_RD7 = (data1 & 0b00001000)>>3; LCD_RD6 = (data1 & 0b00000100)>>2;
LCD_RD5 = (data1 & 0b00000010)>>1; LCD_RD4 = (data1 & 0b00000001);
Cpu_Delay100US(100);
LCD_EN = 1; delay_lcd();  LCD_EN = 0;
delay_lcd(); Cpu_Delay100US(55);
}


void Lcd_Cmd(unsigned char data1){
LCD_EN = 0; LCD_RS = 0;
LCD_RD7 = (data1 & 0b10000000)>>7; LCD_RD6 = (data1 & 0b01000000)>>6;
LCD_RD5 = (data1 & 0b00100000)>>5; LCD_RD4 = (data1 & 0b00010000)>>4;
Cpu_Delay100US(100);
LCD_EN = 1; delay_lcd();  LCD_EN = 0;
LCD_RD7 = (data1 & 0b00001000)>>3; LCD_RD6 = (data1 & 0b00000100)>>2;
LCD_RD5 = (data1 & 0b00000010)>>1; LCD_RD4 = (data1 & 0b00000001);
Cpu_Delay100US(100);
LCD_EN = 1; delay_lcd();  LCD_EN = 0;
Cpu_Delay100US(55);
}

void Lcd_One_Chr(unsigned char P, unsigned char L, char chr){
	unsigned char data1;
	switch (P)
	{
	    case 1: data1 = 0x80 + L; break;
	    case 2: data1 = 0xC0 + L; break;
	    case 3: data1 = 0x94 + L; break;
	    case 4: data1 = 0xD4 + L; break;
	    default: break;
	}
	Lcd_Cmd(data1);
	Lcd_Chr_CP(chr);
}
